/* L-Dic(Linux Dictionary)
 * Copyright (C) 2002 Baek SeungSoon.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <malloc.h>
#include <string.h>
#include <getopt.h>

#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include "ldic.h"
#include "wlist.h"
#include "rcfile.h"
#include "readdic.h"
#include "fileops.h"
#include "tagtrans.h"

#include "window.h"

/* dicLoad:   б
 */
FILE
*dicLoad(char *dicName)
{
	FILE *fp;
	
	fp = fopen(dicName, "r");
	if (fp == NULL) {
		fprintf(stderr, "Error: %s, %s\n", dicName, getErrMsg("dic"));
		return NULL;
	}

	if (isDic(fp) == 0) {
		fprintf(stderr, "Error: %s, %s\n", dicName, getErrMsg("notdic"));
		return NULL;
	}
	
	return fp;
}

/* fileSearch: Ͽ ܾ ˻
 */
void
fileSearch(char *word)
{
	int i;
	
	for (i = 0; i < DICTIONARY_MAX; i++) {
		FILE *fp;

		if (dic_list[i] == NULL)
			continue;
	
		fp = dicLoad(dic_list[i]);
		if (fp == NULL) {
			continue;
		} else {
			long offset;
			char *s;
			
			printf("%s ˻..\n", dic_list[i]);

			offset = wordSearch(word, fp);
			if (offset == -1) {
				fclose(fp);
				continue;
			}
			
			s = offsetSearch(offset, fp);
			fclose(fp);
			
			printMean(s);

			free(s);
			break;
		}
	}
	
	if (i == DICTIONARY_MAX) {
		fprintf(stderr, "Error: %s\n", getErrMsg("word"));
	}
}

/* commandLine: Console ܾ  ˻
 */
void
commandLine(void)
{
	int i;
	WList *wlist = NULL;
	char word[BUFSIZ+1];
	FILE *fp;
	
	write(1, "Words List Loding...\t\t", 23);
	for (i = 0; i < DICTIONARY_MAX; i++) {
		if (dic_list[i] == NULL) {
			continue;
		}
		
		fp = dicLoad(dic_list[i]);
		if (fp == NULL) {
			continue;
		}
		
		wlist = memList(wlist, i, fp);
		fclose(fp);
	}
	printf("[Done]\n");

	for (;;) {
		WList *temp = NULL;
		
		write(1, "Search Word ('/exit'): ", 23);
		
		fgets(word, sizeof(word), stdin);
		word[strlen(word)-1] = '\0';

		if (word[0] == '/') {
			if(strcmp(word, "/exit") == '\0') {
				break;
			} else {
				printf("Error: %s\n", getErrMsg("cmd"));
			}
			continue;
		}

		temp = ListSearchWord(wlist, word);
		if (temp != NULL) {
			char *s;
			fp = dicLoad(dic_list[temp->num]);
			s = offsetSearch(temp->offset, fp);
			printMean(s);
			fclose(fp);
		} else {
			printf("Error: %s\n", getErrMsg("word"));
		}
	}

	freeall(wlist);
}

/* printMean: ܾ  
 */
void
printMean(char *s)
{
	int i;
	char **tagtext;
	
	tagtext = TagAndTextList(s);
	printf("[_mean_]\n");
	for (i = 0; tagtext[i] != NULL; i++) {
		if (tagtext[i][0] == '<' || (tagtext[i][0] == '&' && tagtext[i][1] != '[')) {
			if (strcmp(tagtext[i], "<br>") == '\0') {
				printf("\n");
			} else if (strncmp(tagtext[i], "&lt;", 4) == '\0') {
				printf("<");
			} else if (strncmp(tagtext[i], "&gt;", 4) == '\0') {
				printf(">");
			} else if (strncmp(tagtext[i], "&nbsp;", 4) == '\0') {
				printf(" ");
			} else if (strncmp(tagtext[i], "<img src", 8) == '\0') {
				printf("_");
			}
		} else {
			printf("%s", tagtext[i]);
		}
	}
	printf("\n");
}

/* getErrMsg: err شϴ ޼ ȯ
 */
char
*getErrMsg(char *err)
{
	static char retval[BUFSIZ+1];
	
	char *errList[] = {
		"arg",    "Էµ ܾ ϴ.",
		"list",   " ã  ϴ.",
		"dic",    " ã  ϴ.",
		"notdic", "   Դϴ.",
		"word",   "ܾ ã  ϴ.",
		"cmd",    "ӵ  ƴմϴ.",
		NULL
	};

	int i;

	for (i = 0; errList[i] != NULL; i += 2) {
		if (strcmp(err, errList[i]) == '\0') {
			strcpy(retval, errList[i+1]);
			break;
		}
	}

	if (errList[i] == NULL) {
		strcpy(retval, "    ߻߽ϴ.");
	}

	return retval;
}

/* usage: α׷  
 */
void
usage(const char *str)
{
	char *s;
	
	if ((s = strrchr(str, '/')) != NULL) {
		printf(": %s [ɼ]... [ܾ]\n", s+1);
	} else {
		printf(": %s [ɼ]... [ܾ]\n", str);
	}

	printf("\n");
	printf("  -c,     \tӰ˻\n");
	printf("  -q WORD,\t˻  \n");
	printf("  -d FILE,\t \n");
	printf("  -h,     \t   \n");
	printf("  -v,     \tα׷    \n");
	printf("\n");
	printf("<tadboy@ldic.kldp.net>    ּ\n");
}

/* version: α׷  
 */
void
version(void)
{
	printf("L-Dic(Linux Dictionary) Version %s\n", VERSION);
	printf("¼  ϴ.\n\n");
	printf("Copyright (C) 2002 Free Software Foundation, Inc.\n");
	printf(" α׷  ƮԴϴ. "
		"  ҽ Ͻʽÿ. "
		"ǰ̳ Ư   ռ Ͽ, "
		"   ʽϴ.\n");
}

/* main: α׷ ϰ, ɼ ó
 */
int
main(int argc, char *argv[])
{
	char c;
	char word[BUFSIZ+1];
	char *dicName = NULL;

	int cmdOpt = 0;		/* Ӱ˻ ɼ */
	int quickOpt = 0;	/* ˻ ɼ */
	int dbOpt = 0;		/*   */

	while ((c =  getopt(argc, argv,	"cqd:h?v")) != -1) {
		switch (c) {
			case 'c':
				cmdOpt = 1;
				break;
			case 'q':
				quickOpt = 1;
				break;
			case 'd':
				dbOpt = 1;
				dicName = (char *)malloc(strlen(optarg)+1);
				strcpy(dicName, optarg);
				break;
			case 'h':
			case '?':
				usage(argv[0]);
				exit(0);
			case 'v':
				version();
				exit(0);
			default:
				break;
		}
	}

	if (optind >= argc && quickOpt) {
		fprintf(stderr, "Error: %s\n", getErrMsg("arg"));
		usage(argv[0]);
		exit(1);
	} else if (optind < argc) {
		quickOpt = 1;
	}

	for (; optind < argc; ++optind) {
		strcat(word, argv[optind]);
	}

	check_for_home_path(LDIC_RC_DIR);
	
	load_options();

	if (dicName != NULL) {
		if (dic_list[0] != NULL) {
			free(dic_list[0]);
		}
		dic_list[0] = (char *)malloc(strlen(dicName)+1);
		strcpy(dic_list[0], dicName);
		free(dicName);
	}

	if (quickOpt == 1) {
		fileSearch(word);
	} else if (cmdOpt == 1 || dicName != NULL) {
		commandLine();
	} else {
		gtk_set_locale();
		gtk_init(&argc, &argv);
		
		create_window();
		diclist_refresh();
		wordlist_refresh();

		gtk_main();
	}

	return 0;
}
