/* L-Dic(Linux Dictionary)
 * Copyright (C) 2002 Baek SeungSoon.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "ldic.h"
#include "rcfile.h"
#include "fileops.h"

extern void
save_options(void)
{
	int i;
	FILE *fp;
	char buf[BUFSIZ+1];

	sprintf(buf, "%s/%s/%s", homedir(), LDIC_RC_DIR, RC_FILE_NAME);

	fp = fopen(buf, "w");
	if (fp == NULL) {
		printf("error saving config file: %s\n", buf);
		free(buf);
		return;
	}

	fprintf(fp, "####################################\n");
	fprintf(fp, "# LDic config file version %7s #\n", VERSION);
	fprintf(fp, "####################################\n");
	fprintf(fp, "\n");

	fprintf(fp, "# Dictionary file list #\n");
	for (i = 0; i < DICTIONARY_MAX; i++) {
		if (dic_list[i] == NULL)
			fprintf(fp, "dictionary%d=\n", i);
		else
			fprintf(fp, "dictionary%d=%s\n", i, dic_list[i]);
	}
	fprintf(fp, "\n");
	
	fprintf(fp, "# GUI Options #\n");
	fprintf(fp, "dic_num=%d\n", dic_num);
	fprintf(fp, "startup_position=%d\n", startup_position);
	fprintf(fp, "main_window_x=%d\n", main_window_x);
	fprintf(fp, "main_window_y=%d\n", main_window_y);
	fprintf(fp, "main_window_w=%d\n", main_window_w);
	fprintf(fp, "main_window_h=%d\n", main_window_h);
	fprintf(fp, "\n");

	fprintf(fp, "###########################\n");
	fprintf(fp, "# End of LDic config file #\n");
	fprintf(fp, "###########################\n");

	fclose(fp);
}

extern void
load_options(void)
{
	FILE *fp;
	char buf[1024];
	char *option;
	char *value;

	sprintf(buf, "%s/%s/%s", homedir(), LDIC_RC_DIR, RC_FILE_NAME);

	fp = fopen(buf, "r");
	if (fp == NULL) {
		return;
	}

	while (fgets(buf, 1024, fp)) {
		char *s;
		if (buf[0] < 'a' && buf[0] > 'z')
			continue;

		buf[strlen(buf)-1] = '\0';
		s = strchr(buf, '=');
		if (s == NULL)
			continue;
		
		if (*(s+1) == '\0')
			continue;

		value = (char *)malloc(strlen(s+1)+1);
		strcpy(value, s+1);
		
		*s = '\0';
		option = (char *)malloc(strlen(buf)+1);
		strcpy(option, buf);

		if (value[0] == '~') {
			sprintf(buf, "%s%s", homedir(), value+1);
			free(value);
			value = (char *)malloc(strlen(buf)+1);
			strcpy(value, buf);
		}

		if (strncmp(option, "dictionary", 10) == '\0') {
			int num;
			num = atoi(option+10);
			if (num < DICTIONARY_MAX && value != NULL) {
				dic_list[num] = (char *)malloc(strlen(value)+1);
				strcpy(dic_list[num], value);
			}
		}
		
		if (strcmp("dic_num", option) == '\0')
			dic_num = atoi(value);
		if (strcmp("startup_position", option) == '\0')
			startup_position = atoi(value);
		if (strcmp("main_window_x", option) == '\0')
			main_window_x = atoi(value);
		if (strcmp("main_window_y", option) == '\0')
			main_window_y = atoi(value);
		if (strcmp("main_window_w", option) == '\0')
			main_window_w = atoi(value);
		if (strcmp("main_window_h", option) == '\0')
			main_window_h = atoi(value);

		free(option);
		free(value);
	}
}
