/* L-Dic(Linux Dictionary)
 * Copyright (C) 2002 Baek SeungSoon.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "readdic.h"
#include "wlist.h"

#define DIC_NAME	"LDIC"
#define DIC_NAME_LEN	4

/* isDic: ldic   ´ ˻
 */
extern int
isDic(FILE *fp)
{
	char buf[DIC_NAME_LEN + 1];
	
	fseek(fp, 0, SEEK_SET);
	fread(buf, DIC_NAME_LEN, 1, fp);
	buf[DIC_NAME_LEN] = '\0';

	if (strcmp(buf, DIC_NAME) != '\0')
		return 0; /* ldic   ƴ. */
	else
		return 1; /* ldic  . */
}

/* readHeader: fp header ȯ
 */
extern struct dic_t
readHeader(FILE *fp)
{
	struct dic_t dic;

	fseek(fp, DIC_NAME_LEN, SEEK_SET);
	fread(&dic, sizeof(struct dic_t), 1, fp);

	return dic;
}

/* memList: dic word wlist ߰
 */
extern WList
*memList(WList *wlist, int num, FILE *fp)
{
	struct dic_t dic;

	dic = readHeader(fp);

	fseek(fp, dic.l_offset_s, SEEK_SET);
	while (ftell(fp) < dic.l_offset_e) {
		long offset;
		long length;
		char *buf;

		fread(&offset, sizeof(long), 1, fp);
		fread(&length, sizeof(long), 1, fp);
		buf = (char *)malloc(length+1);
		fread(buf, length, 1, fp);
		buf[length] = '\0';
		wlist = addfront(wlist, newitem(num, offset, buf));
		free(buf);
	}

	return wlist;
}

/* wordSearch: dic word ˻  offset ȯ
 */
extern long
wordSearch(const char *word, FILE *fp)
{
	struct dic_t dic;

	dic = readHeader(fp);

	fseek(fp, dic.l_offset_s, SEEK_SET);
	while (ftell(fp) < dic.l_offset_e) {
		long offset;
		long length;
		char *buf;

		fread(&offset, sizeof(long), 1, fp);
		fread(&length, sizeof(long), 1, fp);
		buf = (char *)malloc(length+1);
		fread(buf, length, 1, fp);
		buf[length] = '\0';

		if (strcmp(buf, word) == '\0') {
			free(buf);
			return offset;
		}
		free(buf);
	}

	return -1; /* ã  */
}

/* offsetSearch: fp offset ġ   ȯ
 */
extern char
*offsetSearch(long offset, FILE *fp)
{
	long length;
	static char *buf;

	struct dic_t dic;

	dic = readHeader(fp);

	fseek(fp, offset + dic.m_offset_s, SEEK_SET);
	fread(&length, sizeof(long), 1, fp);
	buf = (char *)malloc(length+1);
	fread(buf, length, 1, fp);
	buf[length] = '\0';

	return buf;
}

/* readFnt: fp name ˻  image ̸ ȯ
 */
extern struct fnt_t
readFnt(const char *name, FILE *fp)
{
	struct dic_t dic;
	struct fnt_t fnt;

	dic = readHeader(fp);

	if (dic.font == 0) {
		return fnt;
	}

	fseek(fp, dic.f_offset_s, SEEK_SET);
	while (ftell(fp) < dic.f_offset_e) {
		char *n;
		long length;

		fread(&length, sizeof(long), 1, fp);
		n = (char *)malloc(length+1);
		fread(n, length, 1, fp);
		n[length] = '\0';

		fread(&fnt.length, sizeof(long), 1, fp);
		if (strcmp(n, name) == '\0') {
			fnt.image = (char *)malloc(fnt.length);
			fread(fnt.image, fnt.length, 1, fp);
			break;
		} else {
			free(fnt.image);
			fnt.image = NULL;
			fseek(fp, length, SEEK_CUR);
		}
	}

	return fnt;
}
