/* L-Dic(Linux Dictionary)
 * Copyright (C) 2002 Baek SeungSoon.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <stdio.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include "ldic.h"
#include "intl.h"
#include "window.h"
#include "readdic.h"
#include "fileops.h"
#include "ui_preferences.h"

GtkWidget *dictionary_clist;

/* config memory values */
static gint startup_position_c;

static GtkWidget *filesel_dlg;

static void
dictionary_refresh(void)
{
	gint i;
	gint row;
	gchar *buf[2];
	
	gtk_clist_freeze(GTK_CLIST(dictionary_clist));
	gtk_clist_clear(GTK_CLIST(dictionary_clist));
	for (i = 0; i < DICTIONARY_MAX; i++) {
		if (dic_list[i] == NULL)
			continue;
		buf[0] = g_strdup(dic_list[i]);
		buf[1] = NULL;
		row = gtk_clist_append(GTK_CLIST(dictionary_clist), buf);
	}
	gtk_clist_thaw(GTK_CLIST(dictionary_clist));
}

static void
dialog_ok_cb(GtkWidget *widget, gpointer data)
{
	gint i;
	gchar *buf;
	buf = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel_dlg)));
	if (isdir(buf)) {
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel_dlg), buf);
		return;
	}
	
	for (i = 0; i < DICTIONARY_MAX; i++) {
		if (dic_list[i] == NULL) {
			FILE *fp;
			
			fp = fopen(buf, "r");
			if (fp == NULL)
				return;
			if (isDic(fp) == 1) {
				dic_list[i] = g_strdup(buf);
			}
			g_free(buf);
			fclose(fp);
			break;
		}
	}
	dictionary_refresh();
	gtk_widget_destroy(widget);
}

static void
dialog_close_cb(GtkWidget *widget, gpointer data)
{
	gtk_widget_destroy(widget);
}

static void
filesel_dialog_create(void)
{
	filesel_dlg = gtk_file_selection_new("Select a dictionary file");
	gtk_signal_connect(GTK_OBJECT(filesel_dlg), "delete_event",
			GTK_SIGNAL_FUNC(dialog_close_cb), NULL);
	gtk_signal_connect_object(
			GTK_OBJECT(GTK_FILE_SELECTION(filesel_dlg)->ok_button),
			"clicked", GTK_SIGNAL_FUNC(dialog_ok_cb), GTK_OBJECT(filesel_dlg));
	gtk_signal_connect_object(
			GTK_OBJECT(GTK_FILE_SELECTION(filesel_dlg)->cancel_button),
			"clicked", GTK_SIGNAL_FUNC(gtk_widget_hide), GTK_OBJECT(filesel_dlg));
	gtk_widget_show(filesel_dlg);
}

static void
dictionary_add_cb(GtkWidget *widget, gpointer data)
{
	filesel_dialog_create();
}

/* -------------
 * config window
 * -------------
 */
static GtkWidget *configwindow = NULL;

static void
config_window_apply(void)
{
	startup_position = startup_position_c;
	diclist_refresh();
}

static void
config_window_close_cb(GtkWidget *widget, gpointer data)
{
	gtk_widget_destroy(configwindow);
	configwindow = NULL;
}

static void
config_window_destroy(GtkWidget *w, GdkEvent *event, gpointer data)
{
	config_window_close_cb(NULL, NULL);
}

static void
config_window_ok_cb(GtkWidget *widget, gpointer data)
{
	config_window_apply();
	config_window_close_cb(NULL, NULL);
}

static void
config_window_apply_cb(GtkWidget *widget, gpointer data)
{
	config_window_apply();
}

static void
check_button_cb(GtkWidget *widget, gpointer data)
{
	gint *value_ptr = data;
	*value_ptr = GTK_TOGGLE_BUTTON(widget)->active;
}

static void
add_check_button(gint option, gint *option_c, gchar *text, GtkWidget *box)
{
	GtkWidget *button;
	*option_c = option;
	button = gtk_check_button_new_with_label(text);
	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), option);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			(GtkSignalFunc)check_button_cb, option_c);
	gtk_widget_show(button);
}

static void
config_window_create()
{
	GtkWidget *win_vbox;
	GtkWidget *hbox;
	GtkWidget *notebook;
	GtkWidget *frame;
	GtkWidget *label;
	GtkWidget *vbox;
	GtkWidget *vbox1;
	GtkWidget *button;
	GtkWidget *scrolled;

	configwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect(GTK_OBJECT(configwindow), "delete_event",
			(GtkSignalFunc)config_window_destroy, NULL);
	gtk_window_set_policy(GTK_WINDOW(configwindow), FALSE, FALSE, FALSE);
	gtk_window_set_title(GTK_WINDOW(configwindow), _("LDic configuration"));
	gtk_window_set_wmclass(GTK_WINDOW(configwindow), "config", "LDic");
	gtk_container_border_width(GTK_CONTAINER(configwindow), 5);

	win_vbox = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(configwindow), win_vbox);
	gtk_widget_show(win_vbox);

	hbox = gtk_hbutton_box_new();
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 0);
	gtk_button_box_set_child_size(GTK_BUTTON_BOX(hbox), 95, 40);
	gtk_box_pack_end(GTK_BOX(win_vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	button = gtk_button_new_with_label(_("Ok"));
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			(GtkSignalFunc) config_window_ok_cb, NULL);
	gtk_container_add(GTK_CONTAINER(hbox), button);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(button);
	gtk_widget_show(button);

	button = gtk_button_new_with_label(_("Apply"));
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			(GtkSignalFunc) config_window_apply_cb, NULL);
	gtk_container_add(GTK_CONTAINER(hbox), button);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_widget_show(button);
	
	button = gtk_button_new_with_label(_("Cancel"));
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			(GtkSignalFunc) config_window_close_cb, NULL);
	gtk_container_add(GTK_CONTAINER(hbox), button);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_widget_show(button);

	notebook = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
	gtk_box_pack_start(GTK_BOX(win_vbox), notebook, TRUE, TRUE, 0);

	/* general options tab */
	frame = gtk_frame_new(NULL);
	gtk_container_border_width(GTK_CONTAINER(frame), 5);
	gtk_widget_show(frame);
	label = gtk_label_new(_("General"));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_border_width(GTK_CONTAINER(vbox), 5);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_widget_show(vbox);
	
	frame = gtk_frame_new(_("Startup Position"));
	gtk_container_border_width(GTK_CONTAINER(frame), 5);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), vbox1);
	gtk_widget_show(vbox1);

	add_check_button(startup_position, &startup_position_c,
			_(" ġ ϱ"), vbox1);
	
	frame = gtk_frame_new(_(""));
	gtk_container_border_width(GTK_CONTAINER(frame), 5);
	gtk_box_pack_end(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), vbox1);
	gtk_widget_show(vbox1);
	
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 0);
	gtk_widget_show(scrolled);

	dictionary_clist = gtk_clist_new(1);
	gtk_container_add(GTK_CONTAINER(scrolled), dictionary_clist);
	gtk_widget_show(dictionary_clist);

	button = gtk_button_new_with_label(_("߰ϱ"));
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			(GtkSignalFunc)dictionary_add_cb, NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	gtk_widget_show(notebook);
	
	gtk_widget_show(configwindow);
}

extern void
show_config_window(void)
{
	if (configwindow != NULL)
		return;

	config_window_create();
	dictionary_refresh();
}

/* ------------
 * about window
 * ------------
 */
static GtkWidget *about = NULL;

static gint
about_delete_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
{
	gtk_widget_destroy(about);
	about = NULL;

	return TRUE;
}

extern void
show_about_window(void)
{
	GtkWidget *frame;
	GtkWidget *vbox;
	GtkWidget *label;

	gchar *buf;

	if (about) {
		gdk_window_raise(about->window);
		return;
	}

	about = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_wmclass(GTK_WINDOW(about), "about", "LDic");
	gtk_signal_connect(GTK_OBJECT(about), "delete_event",
			GTK_SIGNAL_FUNC(about_delete_cb), NULL);

	gtk_window_set_title(GTK_WINDOW(about), "About - LDic");
	gtk_container_border_width(GTK_CONTAINER(about), 5);

	frame = gtk_frame_new(NULL);
	gtk_container_add(GTK_CONTAINER(about), frame);
	gtk_widget_show(frame);

	vbox = gtk_vbox_new(FALSE, 10);
	gtk_container_border_width(GTK_CONTAINER(vbox), 5);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_widget_show(vbox);

	buf = g_strdup_printf(_("Linux Dictionary (LDic) %s\n\n"
				"Copyright (c) 2002 by Baek SeungSoon\n"
				"http://ldic.kldp.net\n"
				"tadboy@ldic.kldp.net\n\n"
				"Released under the GNU General Public License"), VERSION);
	label = gtk_label_new(buf);
	g_free(buf);

	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

	gtk_widget_show(about);
}
