/* L-Dic(Linux Dictionary)
 * Copyright (C) 2002 Baek SeungSoon.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk-pixbuf/gdk-pixbuf-loader.h>

#include "intl.h"
#include "window.h"
#include "ui_tools.h"
#include "ui_preferences.h"

#include "icons/icon_prev.xpm"
#include "icons/icon_next.xpm"
#include "icons/icon_reload.xpm"
#include "icons/icon_config.xpm"

/* menu and button callbacks (private)
 */
static void
exit_cb(void)
{
	quit_window();
}

static void
about_cb(void)
{
	show_about_window();
}

static void
prev_cb(void)
{
	if (wordlist_select_row > 0)
		scroll_and_select_list(wordlist_select_row-1);
}

static void
next_cb(void)
{
	if (wordlist_select_row < GTK_CLIST(word_clist)->rows-1)
		scroll_and_select_list(wordlist_select_row+1);
}

static void
refresh_cb(void)
{
	wordlist_refresh();
}

static void
config_cb(void)
{
	show_config_window();
}

/* menubar
 */
GtkItemFactory *ldic_fact;

static GtkItemFactoryEntry ldic_menu_items[] = {
	{ N_("/_File"),			NULL,		NULL,		0, "<Branch>" },
	{ N_("/File/tear1"),		NULL,		NULL,		0, "<Tearoff>" },
	{ N_("/File/sep"),		NULL,		NULL,		0, "<Separator>" },
	{ N_("/File/E_xit"),		"<control>Q",	exit_cb,	0, NULL },
	{ N_("/_Edit"),			NULL,		NULL,		0, "<Branch>" },
	{ N_("/Edit/tear1"),		NULL,		NULL,		0, "<Tearoff>" },
	{ N_("/Edit/_Options..."),	"<control>O",	config_cb,	0, NULL },
	{ N_("/_View"),			NULL,		NULL,		0, "<Branch>" },
	{ N_("/View/tear1"),		NULL,		NULL,		0, "<Tearoff>" },
	{ N_("/View/_Refresh Lists"),	"R",		refresh_cb,	0, NULL },
	{ N_("/_Help"),			NULL,		NULL,		0, "<Branch>" },
	{ N_("/Help/tear1"),		NULL,		NULL,		0, "<Tearoff>" },
	{ N_("/Help/_About"),		NULL,		about_cb,	0, NULL },
};

static int ldic_nmenu_items = sizeof (ldic_menu_items) / sizeof (ldic_menu_items[0]);

static gchar
*menu_translate(const gchar *path, gpointer data)
{
	return (gchar *)_(path);
}

extern GtkWidget
*create_menu_bar(GtkAccelGroup *accel_grp)
{
	ldic_fact = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel_grp);
	gtk_item_factory_set_translate_func(ldic_fact, menu_translate, NULL, NULL);

	gtk_item_factory_create_items(ldic_fact, ldic_nmenu_items, ldic_menu_items, NULL);

	return gtk_item_factory_get_widget(ldic_fact, "<main>");
}

/* toolbar
 */
GtkWidget *thumb_button;

static void add_button_to_bar(GtkWidget *hbox, gchar **pixmap_data,
		GtkTooltips *tooltips, gchar *tip_text,
		GtkSignalFunc func, gpointer data)
{
	GtkWidget *button;
	GtkWidget *pixmapwid;
	GdkPixbuf *pixbuf;
	GdkPixmap *pixmap;
	GdkBitmap *mask;

	button = gtk_button_new();
	gtk_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc)func, thumb_button);
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);
	gtk_tooltips_set_tip(tooltips, button, tip_text, NULL);

	pixbuf = gdk_pixbuf_new_from_xpm_data((const char **)pixmap_data);
	gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pixmap, &mask, 128);
	gdk_pixbuf_unref(pixbuf);

	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
	gtk_container_add(GTK_CONTAINER(button), pixmapwid);

	gdk_pixmap_unref(pixmap);
	if (mask)
		gdk_bitmap_unref(mask);
}

extern GtkWidget
*create_button_bar(GtkTooltips *tooltips)
{
	GtkWidget *hbox;
	GtkStyle *style;

	style = gtk_widget_get_style(mainwindow);
	hbox = gtk_hbox_new(FALSE, 0);

	add_button_to_bar(hbox, (gchar **)icon_prev_xpm, tooltips,
			_(" ܾ"), prev_cb, NULL);
	add_button_to_bar(hbox, (gchar **)icon_next_xpm, tooltips,
			_(" ܾ"), next_cb, NULL);
	add_button_to_bar(hbox, (gchar **)icon_reload_xpm,
			tooltips, _(" ٽ б"), refresh_cb, NULL);
	add_button_to_bar(hbox, (gchar **)icon_config_xpm,
			tooltips, _("ȯἳ"), config_cb, NULL);

	return hbox;
}
