/* L-Dic(Linux Dictionary)
 * Copyright (C) 2002 Baek SeungSoon.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include "ldic.h"
#include "rcfile.h"
#include "window.h"
#include "readdic.h"
#include "tagtrans.h"

#include "ui_tools.h"

WList *wlist;

gint wordlist_select_row = -1;

static void
load_word_list(void)
{
	FILE *fp;

	fp = fopen(dic_list[dic_num], "r");
	if (fp == NULL)
		return;
	if (isDic(fp) == FALSE)
		return;
	
	wlist = NULL;
	wlist = memList(wlist, dic_num, fp);
	fclose(fp);
}

extern GdkColor
str_rgb_set(gchar *str)
{
	long c;
	GdkColor color;
	
	c = strtol(str, NULL, 16);
	color.red = (c / 0x10000) * 0x100;
	color.green = ((c % 0x10000) / 0x100) * 0x100;
	color.blue = (c % 0x100) * 0x100;

	return color;
}

static GdkColor
tag_color_set(gchar *tag)
{
	gchar *s;
	gchar *e;
	
	GdkColor color;

	s = strchr(tag, '=');
	s++;
	e = strrchr(tag, '>');
	if (s == NULL || e == NULL) {
		gdk_color_parse("Black", &color);
		return color;
	}

	*e = '\0';
	if (s[0] == '#') {
		s++;
		color = str_rgb_set(s);
	} else {
		gdk_color_parse(s, &color);
	}
	
	return color;
}

static void
mean_to_text(const gchar *word)
{
	gint i;
	FILE *fp;
	WList *temp;
	gchar *mean;

	gchar **tagtext;

	GdkColor text_color;

	gtk_text_freeze(GTK_TEXT(text));
	gtk_editable_delete_text(GTK_EDITABLE(text), 0, -1);
	
	temp = ListSearchWord(wlist, word);
	if (temp == NULL)
		return;
	fp = fopen(dic_list[temp->num], "r");
	if (fp == NULL || isDic(fp) == 0)
		return;
	mean = (gchar *)offsetSearch(temp->offset, fp);
	fclose(fp);

	tagtext = TagAndTextList(mean);
	gdk_color_parse("Black", &text_color);

	for (i = 0; tagtext[i] != NULL; i++) {
		char *buf = NULL;
		
		if (tagtext[i][0] == '<' || (tagtext[i][0] == '&' && tagtext[i][1] != '[')) {
			if (strcmp(tagtext[i], "<br>") == '\0') {
				buf = g_strdup("\n");
			} else if (strcmp(tagtext[i], "&lt;") == '\0') {
				buf = g_strdup("<");
			} else if (strcmp(tagtext[i], "&gt;") == '\0') {
				buf = g_strdup(">");
			} else if (strcmp(tagtext[i], "&nbsp;") == '\0') {
				buf = g_strdup(" ");
			} else if (strncmp(tagtext[i], "<img src", 8) == '\0') {
				buf = g_strdup("_");
			} else if (strncmp(tagtext[i], "<font color", 11) == '\0') {
				text_color = tag_color_set(tagtext[i]);
			} else if (strcmp(tagtext[i], "</font>") == '\0') {
				gdk_color_parse("Black", &text_color);
			}
		} else {
			buf = g_strdup(tagtext[i]);
		}

		if (buf != NULL) {
			gdk_colormap_alloc_color(gdk_colormap_get_system(), &text_color, FALSE, TRUE);
			gtk_text_insert(GTK_TEXT(text),
				text->style->font, &text_color, NULL, buf, strlen(buf));

			g_free(buf);
		}
	}
	
	gtk_text_thaw(GTK_TEXT(text));
}

static gint
find_word_in_list(const gchar *word)
{
	WList *work = wlist;
	gint count = -1;

	if (!word)
		return -1;

	while (work) {
		count++;
		if (strncasecmp(work->word, word, strlen(word)) == '\0')
			return count;
		work = work->next;
	}

	return -1;
}

static void 
mainwindow_delete_cb(GtkWidget *widget, gpointer data)
{
	quit_window();
}

extern void
scroll_and_select_list(gint row)
{
	GtkAdjustment *adj;
	gint rows;
	gfloat value;
	gfloat small_val;
	gfloat large_val;
	
	adj = GTK_CLIST(word_clist)->vadjustment;
	rows = GTK_CLIST(word_clist)->rows;

	small_val = adj->value;
	large_val = adj->value + adj->page_increment;

	value = adj->upper * ((float)(row - 1) / (float)(rows - 1));

	gtk_clist_freeze(GTK_CLIST(word_clist));
	
	if (value < small_val || value > large_val)
		gtk_adjustment_set_value(GTK_CLIST(word_clist)->vadjustment, value);

	gtk_clist_select_row(GTK_CLIST(word_clist), row, 0);
	gtk_clist_thaw(GTK_CLIST(word_clist));
}

static void
word_entry_changed_cb(GtkWidget *widget, gpointer data)
{
	gint i;
	gint n;
	gint row;
	gchar *buf;
	gchar *nospace;

	buf = g_strdup(gtk_entry_get_text(GTK_ENTRY(word_entry)));
	if (!buf || strlen(buf) < 1) {
		//gtk_editable_delete_text(GTK_EDITABLE(text), 0, -1);
		g_free(buf);
		return;
	}

	/*  Էµ , ˻ ʴ´. */
	if (buf[strlen(buf)-1] == ' ') {
		g_free(buf);
		return;
	}

	/*  ϰ,  ܾ ģ. */
	nospace = (gchar *)malloc(strlen(buf)+1);
	n = 0;
	for (i = 0; buf[i] != '\0'; i++) {
		if (buf[i] != ' ') {
			nospace[n] = buf[i];
			n++;
		}
	}
	nospace[n] = '\0';
	if (n == 0)
		return;
	
	/* search word */
	row = find_word_in_list(nospace);

	if (row != -1)
		scroll_and_select_list(row);

	g_free(buf);
}

static void
word_entry_press_cb(GtkWidget *widget, gpointer data)
{
	gchar *buf;
	
	buf = g_strdup(gtk_entry_get_text(GTK_ENTRY(word_entry)));
	gtk_entry_select_region(GTK_ENTRY(word_entry), 0, strlen(buf));

	g_free(buf);
}

static void
word_select_cb(GtkWidget *widget, gint row, gint col,
		GdkEvent *event, gpointer data)
{
	gchar *word;

	wordlist_select_row = row;

	word = g_strdup(gtk_clist_get_row_data(GTK_CLIST(word_clist), row));
	mean_to_text(word);
	gtk_widget_grab_focus(word_entry);

	g_free(word);
}

static void
dic_select_cb(GtkWidget *widget, gpointer data)
{
	gint i;
	for (i = 0; i < DICTIONARY_MAX; i++) {
		char *s;
		if (dic_list[i] == NULL)
			continue;
		
		s = strrchr(dic_list[i], '/');
		if (s == NULL) {
			s = dic_list[i];
		} else {
			s++;
		}
		
		if (strcmp(s, data) == '\0')
			break;
	}

	if (i == DICTIONARY_MAX || i == dic_num)
		return; /*   ãҰų,    */

	dic_num = i;
	wordlist_refresh();
	gtk_entry_set_text(GTK_ENTRY(word_entry), "");
	gtk_widget_grab_focus(word_entry);
}

/* wordlist refresh
 */
extern void
wordlist_refresh(void)
{
	gint row;
	gchar *buf[2];
	WList *temp;

	load_word_list();

	gtk_clist_freeze(GTK_CLIST(word_clist));
	gtk_clist_clear(GTK_CLIST(word_clist));
	
	temp = wlist;
	for (; temp != NULL; temp = temp->next) {
		buf[0] = g_strdup(temp->word);
		buf[1] = NULL;
		row = gtk_clist_append(GTK_CLIST(word_clist), buf);
		gtk_clist_set_row_data(GTK_CLIST(word_clist), row, temp->word);
	}
	gtk_clist_set_column_width(GTK_CLIST(word_clist), 0, 10);
	gtk_clist_thaw(GTK_CLIST(word_clist));
	gtk_widget_grab_focus(word_entry);
}

/* diclist refresh
 */
extern void
diclist_refresh(void)
{
	gchar *s;
	GtkWidget *menu;
	GtkWidget *item;
	GdkFont *font;
	int i;
	
	font = dic_select->style->font;

	menu = gtk_menu_new();

	if (dic_list[dic_num] == NULL)
		dic_num = 0;

	for (i = 0; i < DICTIONARY_MAX; i++) {
		if (dic_list[i] == NULL)
			continue;

		s = strrchr(dic_list[i], '/');
		if (s == NULL) {
			s = dic_list[dic_num];
		} else {
			s++;
		}
		item = gtk_menu_item_new_with_label(s);
		gtk_signal_connect(GTK_OBJECT(item), "activate",
				(GtkSignalFunc)dic_select_cb, s);

		if (i == dic_num) {
			gtk_menu_insert(GTK_MENU(menu), item, 0);
		} else {
			gtk_menu_append(GTK_MENU(menu), item);
		}
		gtk_widget_show(item);
	}

	gtk_option_menu_set_menu(GTK_OPTION_MENU(dic_select), menu);
}

/* create tools
 */
static void
create_tools(GtkAccelGroup *accel_grp, GtkTooltips *tooltips)
{
	GtkWidget *menu_bar;
	GtkWidget *button_bar;

	tool_vbox = gtk_vbox_new(FALSE, 0);

	menu_bar = create_menu_bar(accel_grp);
	gtk_box_pack_start(GTK_BOX(tool_vbox), menu_bar, FALSE, FALSE, 0);
	gtk_widget_show(menu_bar);

	button_bar = create_button_bar(tooltips);
	gtk_box_pack_start(GTK_BOX(tool_vbox), button_bar, FALSE, FALSE, 0);
	gtk_widget_show(button_bar);

	word_entry = gtk_entry_new();
	gtk_signal_connect_after(GTK_OBJECT(word_entry), "changed",
			GTK_SIGNAL_FUNC(word_entry_changed_cb), NULL);
	gtk_signal_connect_after(GTK_OBJECT(word_entry), "button_press_event",
			GTK_SIGNAL_FUNC(word_entry_press_cb), NULL);
	
	gtk_box_pack_start(GTK_BOX(tool_vbox), word_entry, FALSE, FALSE, 0);
	gtk_widget_show(word_entry);
	
	dic_select = gtk_option_menu_new();
	gtk_box_pack_start(GTK_BOX(tool_vbox), dic_select, FALSE, FALSE, 0);
	gtk_widget_show(dic_select);
	
	list_scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(list_scrolled),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(tool_vbox), list_scrolled, TRUE, TRUE, 0);
	gtk_widget_show(list_scrolled);

	word_clist = gtk_clist_new(1);
	gtk_signal_connect(GTK_OBJECT(word_clist), "select_row", (GtkSignalFunc)word_select_cb, NULL);
	gtk_container_add(GTK_CONTAINER(list_scrolled), word_clist);
	gtk_widget_show(word_clist);

	gtk_widget_show(tool_vbox);
}

/* create scroll text
 */
static GtkWidget
*create_scroll_text(void)
{
	GtkWidget *scrolled;
	
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_widget_show(scrolled);

	text = gtk_text_new(NULL, NULL);
	gtk_text_set_editable(GTK_TEXT(text), FALSE);
	gtk_container_add(GTK_CONTAINER(scrolled), text);
	gtk_widget_show(text);

	return scrolled;
}

/* create main window
 */
extern void
create_window(void)
{
	GtkTooltips *tooltips;
	GdkColormap *colormap;
	static GdkColor tooltip_color = {0, 0xffff, 0xf9f9, 0xcbcb}; /* 255 249 203 */

	mainwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_policy(GTK_WINDOW(mainwindow), TRUE, TRUE, FALSE);

	gtk_signal_connect(GTK_OBJECT(mainwindow), "delete_event", (GtkSignalFunc)mainwindow_delete_cb, NULL);

	gtk_window_set_title(GTK_WINDOW(mainwindow), "Linux Dictionary (LDic)");
	gtk_window_set_wmclass(GTK_WINDOW(mainwindow), "ldic", "LDic");
	gtk_container_border_width(GTK_CONTAINER(mainwindow), 0);
	
	gtk_window_set_default_size(GTK_WINDOW(mainwindow), main_window_w, main_window_h);
	gtk_widget_set_uposition(mainwindow, main_window_x, main_window_y);

	gtk_widget_realize(mainwindow);

	mainwindow_accel_grp = gtk_accel_group_new();
	gtk_window_add_accel_group(GTK_WINDOW(mainwindow), mainwindow_accel_grp);

	tooltips = gtk_tooltips_new();
	colormap = gdk_window_get_colormap(mainwindow->window);
	gdk_color_alloc(colormap, &tooltip_color);
	gtk_tooltips_set_colors(tooltips, &tooltip_color, &mainwindow->style->fg[GTK_STATE_NORMAL]);

	create_tools(mainwindow_accel_grp, tooltips);

	meanwindow = create_scroll_text();
	
	mainwindow_vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(mainwindow), mainwindow_vbox);
	gtk_widget_show(mainwindow_vbox);

	mainwindow_hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainwindow_vbox), mainwindow_hbox, TRUE, TRUE, 0);
	gtk_widget_show(mainwindow_hbox);
	gtk_box_pack_start(GTK_BOX(mainwindow_hbox), tool_vbox, FALSE, FALSE, 0);

	gtk_box_pack_end(GTK_BOX(mainwindow_hbox), meanwindow, TRUE, TRUE, 0);
	gtk_widget_show(meanwindow);

	gtk_widget_show(mainwindow);
}

extern void
quit_window(void)
{
	if (startup_position == 1) {
		gdk_window_get_root_origin(mainwindow->window, &main_window_x, &main_window_y);
		gdk_window_get_size(mainwindow->window, &main_window_w, &main_window_h);
	}
	
	save_options();

	gtk_main_quit();
}
